@@ -85,12 +85,12 @@ class xevieExtension(xcffib.Extension):
         buf = io.BytesIO()
         buf.write(struct.pack("=xx2xI", cmap))
         return self.send_request(2, buf, EndCookie, is_checked=is_checked)
-    def Send(self, event, data_type, is_checked=True):
+    def Send(self, event, data_type, _, is_checked=True):
         buf = io.BytesIO()
         buf.write(struct.pack("=xx2x"))
         buf.write(event.pack() if hasattr(event, "pack") else Event.synthetic(*event).pack())
         buf.write(struct.pack("=I", data_type))
-        buf.write(struct.pack("=64x", ))
+        buf.write(struct.pack("=64x", _))
         return self.send_request(3, buf, SendCookie, is_checked=is_checked)
     def SelectInput(self, event_mask, is_checked=True):
         buf = io.BytesIO()
